import * as React from 'react'

import Mark, {MarkProps} from './Mark'
import {selectionIsEmpty, selectionIsBackwards, splitTokensWithOffsets, makeBold} from './utils'

interface TokenProps {
  i: number
  content: string
  wordsToBold?: Array<string>
}

const Token: React.SFC<TokenProps> = props => {
  var bold;

  if(props.wordsToBold && makeBold(props.content,props.wordsToBold)){
    bold = { fontWeight: 'bold'}
  }
  return <span data-i={props.i} style={bold}>{props.content} </span>
}

export interface waiting {
  start: number
  end: number
}
export interface TokenAnnotatorProps {
  style: object
  tokens: string[]
  value: any[]
  waiting: waiting 
  wordsToBold?: Array<string>
  pre: boolean
  showTags: boolean
  onChange: (any) => any
  onClickChange?: (any) => any
  getSpan?: (any) => any
  renderMark?: (props: MarkProps) => JSX.Element
  // determine whether to overwrite or leave intersecting ranges.
}

// TODO: When React 16.3 types are ready, remove casts.
class TokenAnnotator extends React.Component<TokenAnnotatorProps, {}> {
  static defaultProps = {
    renderMark: props => <Mark {...props} />,
    showTags: true,
    pre: true,
  }

  rootRef: any

  constructor(props) {
    super(props)

    this.rootRef = (React as any).createRef()
  }

  componentDidMount() {
    this.rootRef.current.addEventListener('mouseup', this.handleMouseUp)
  }

  componentWillUnmount() {
    this.rootRef.current.removeEventListener('mouseup', this.handleMouseUp)
  }

  handleMouseUp = () => {
    if (!this.props.onChange) return

    const selection = window.getSelection()

    if (selectionIsEmpty(selection)) return

    if (
      !selection.anchorNode.parentElement.hasAttribute('data-i') ||
      !selection.focusNode.parentElement.hasAttribute('data-i')
    ) {
      window.getSelection().empty()
      return false
    }
    
    if (this.props.waiting){
      const { end, start } = this.props.waiting 
      this.handleSplitClick({start, end})
    }

    let start = parseInt(selection.anchorNode.parentElement.getAttribute('data-i'), 10)
    let end = parseInt(selection.focusNode.parentElement.getAttribute('data-i'), 10)

    if (selectionIsBackwards(selection)) {
      ;[start, end] = [end, start]
    }

    end += 1

    if(this.props.pre){
      this.props.onChange({
        value: [
        ...this.props.value,
        this.getSpan({start, end, tokens: this.props.tokens.slice(start, end)}),
      ]});
    } else {
      this.props.onChange({
        value: [
        ...this.props.value,{
          start,
          end,
          tokens: this.props.tokens.slice(start, end),
          tag: 'waiting...',
          special_label: '',
          color: '#84d2ff',
        },
      ],
    waiting: {start, end}})
    }

    window.getSelection().empty()
  }

  handleSplitClick = ({start, end}) => {
    console.log('handleSplitClick');
    // Find and remove the matching split.
    const splitIndex = this.props.value.findIndex(s => s.start === start && s.end === end)
    if (splitIndex >= 0) {
      if(this.props.pre){
        this.props.onChange({value: [
          ...this.props.value.slice(0, splitIndex),
          ...this.props.value.slice(splitIndex + 1),
        ]})
      } else { 
      this.props.onChange({value: [
        ...this.props.value.slice(0, splitIndex),
        ...this.props.value.slice(splitIndex + 1),
      ], waiting: null })
    }
    }
  }

  getSpan = span => {
    if (this.props.getSpan) return this.props.getSpan(span)
    return span
  }

  render() {
    const {tokens, value, style, renderMark} = this.props
    const splits = splitTokensWithOffsets(tokens, value)
    return (
      <div style={style} ref={this.rootRef}>
        {splits.map(
          (split, i) =>
            split.mark ? (
              renderMark({
                key: `${split.start}-${split.end}`,
                ...split,
                onClick: this.handleSplitClick,
                onClickChange: this.props.onClickChange,
                wordsToBold: this.props.wordsToBold,
                showTags: this.props.showTags,
              })
            ) : (
              <Token key={split.i} {...split} wordsToBold={ this.props.wordsToBold} onClickChange={this.props.onClickChange}/>
            )
        )}
      </div>
    )
  }
}

export default TokenAnnotator
